'
' file      : mdm_adc.bas
' author    : Ian Forse
' date      : 02-APR-2004
' purpose   : Analogue read and display on MDM-1
' includes  : none
' notes     : Read ADC, convert to actual voltage, display on LCD & send
'           : via RS232
' compiler  : PICBasic Pro Compiler Ver 2.44

' Copyright (C) 2004 Ian Forse

' This library is free software; you can redistribute it and/or
' modify it under the terms of the GNU Lesser General Public
' License as published by the Free Software Foundation; either
' version 2.1 of the License, or (at your option) any later version.

' This library is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
' Lesser General Public License for more details.

' You should have received a copy of the GNU Lesser General Public
' License along with this library; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
' USA

		z VAR Byte
		c VAR Byte
		B1 VAR BYTE
		x VAR Byte
		p VAR Byte
		seg_a VAR Byte
		seg_b VAR Byte
		a VAR Byte
		b VAR Byte
		v VAR Byte
		mdm_seg_1a VAR Byte
		mdm_seg_1b VAR Byte
		mdm_seg_2a VAR Byte
		mdm_seg_2b VAR Byte
		mdm_seg_3a VAR Byte
		mdm_seg_3b VAR Byte
		mdm_seg_4a VAR Byte
		mdm_seg_4b VAR Byte
		mdm_seg_5a VAR Byte
		mdm_seg_5b VAR Byte
		mdm_cursor_position VAR Byte
		Total VAR WORD
		Value VAR WORD
		
        DEFINE ADC_BITS 10
        DEFINE ADC_CLOCK 3              ' Set clock source (rc = 3)
        DEFINE ADC_SAMPLEUS 10          ' Set sampling time in microseconds
         
		TRISA.0 = 1                     ' Setup bit-0 of PortA as an input
        ADCON1 = %10000010              ' Set PORTA analog and right justify res
		
		TRISC = %10111111       		' Set TX (PortC.6) to output
        SPBRG = 12              		' Set baud rate to 19200
        RCSTA = %10010000       		' Enable serial continuous receive
        TXSTA = %00100100       		' Enable transmit asynchronous mode
		
' LOAD,DATA & CLOCK use Portb,2,4,5 respectively

  	    SYMBOL mdm_pin_load=PORTB.2
  	    SYMBOL mdm_pin_data=PORTB.4
	    SYMBOL mdm_pin_clock=PORTB.5
		
        Gosub mdm_init
		Gosub mdm_clear
		
		
' Main software
Loop:   Total = 0
		For z = 1 to 64					' Average of 64 readings to reduce noise
			ADCIN 0, Value
			Total = Total + Value
		next
		Value = Total >> 6
		Value = Value */ $04E2			' multiply by 4.88 (5/1024)
		c = (Value Dig 3) + "0"
		Gosub mdm_write_1  	   	  		' write 1st digit to display
		Gosub Putchar					' send 1st digit via RS232
		c = "."		  	 	   	  		' insert decimal point after 1st digit
		Gosub mdm_write_2				' write to display between 1st & 2nd digit
		Gosub Putchar					' send DP via RS232
		c = (Value Dig 2) + "0"
		Gosub mdm_write_2	   	  	  	' write 2nd digit to display
		Gosub Putchar					' send 2nd digit via RS232
		c = (Value Dig 1) + "0"
		Gosub mdm_write_3  	 	   	   	' write 3rd digit to display 
		Gosub Putchar					' send 3rd digit via RS232
		c = (Value Dig 0) + "0"
		Gosub mdm_write_4 	  	 		' write 4th digit to display 
		Gosub Putchar	  	  	 		' send 4th digit via RS232
		c = "V"
		Gosub mdm_write_5				' write V  to display 
		Gosub Putchar					' send V via RS232
		c = $0d
		Gosub Putchar					' send Carriage Return via RS232
		c = $0a
		Gosub Putchar					' send Line Feed via RS232
		PAUSE 250
Goto Loop               				' Do it forever


' ----------------------------------------------------------------------------
' Send a character to the USART transmitter
' ----------------------------------------------------------------------------

PutChar: If PIR1.4 = 0 Then PutChar     ' Wait for transmit register empty
         B1 = c
		 TXREG = B1              		' Send character to transmit register
Return                  				' Go back to caller


' ----------------------------------------------------------------------------
' Shift byte x out to the display as serial.  (5 x 8) +  (1 * 3)
' ----------------------------------------------------------------------------

mdm_output_byte:
SHIFTOUT mdm_pin_data,mdm_pin_clock,1,_
	  [mdm_seg_5a,mdm_seg_4a,mdm_seg_3a,mdm_seg_2a,mdm_seg_1a,$00\3]
	  high mdm_pin_load
	  low mdm_pin_load
SHIFTOUT mdm_pin_data,mdm_pin_clock,1,_
	  [mdm_seg_5b,mdm_seg_4b,mdm_seg_3b,mdm_seg_2b,mdm_seg_1b,$01\3]
	  high mdm_pin_load
	  low mdm_pin_load
return


' ----------------------------------------------------------------------------
' Adjust segment values to either all on or all off.
' ----------------------------------------------------------------------------

mdm_all:	if x = 1 then
			   v = %11111111
			else
			   v = %00000000
			endif
			mdm_seg_1a = v
			mdm_seg_1b = v
			mdm_seg_2a = v
			mdm_seg_2b = v
			mdm_seg_3a = v
			mdm_seg_3b = v
			mdm_seg_4a = v
			mdm_seg_4b = v
			mdm_seg_5a = v
			mdm_seg_5b = v
			gosub mdm_output_byte
return


' ----------------------------------------------------------------------------
' write ascii character c to position p.
' ----------------------------------------------------------------------------
 
mdm_write_p:	if p = 1 then
				   seg_a = mdm_seg_1a
				   seg_b = mdm_seg_1b
                endif
                if p = 2 then
				   seg_a = mdm_seg_2a
				   seg_b = mdm_seg_2b
                ENDIF 
                if p = 3 then
				   seg_a = mdm_seg_3a
				   seg_b = mdm_seg_3b
                ENDIF
				if p = 4 then
				   seg_a = mdm_seg_4a
				   seg_b = mdm_seg_4b
                ENDIF
				if p = 5 then
				   seg_a = mdm_seg_5a
				   seg_b = mdm_seg_5b
				endif

	if c = 44 then			' look for ',' == clear dots
		seg_a = seg_a & %11111110
		seg_b = seg_b & %11111110
	else
		c = c & %01111111	' tables are 128 entries only, so clear highest bit
		
' ----------------------------------------------------------------------------
' lookup tables for conversion of ASCII character to LCD segment
' there are 2 tables.
' ----------------------------------------------------------------------------		
		
LOOKUP c,[$00,$E2,$1E,$42,$1C,$4A,$16,$00,$00,$00,$00,$00,$00,$00,$00,$00,_
		  $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,_
		  $00,$08,$40,$32,$7A,$62,$2E,$00,$00,$00,$02,$02,$00,$02,$01,$00,_
		  $FC,$90,$CE,$D8,$B2,$7A,$7E,$D0,$FE,$FA,$00,$01,$00,$0A,$00,$E1,_
		  $EC,$F6,$D8,$6C,$D8,$6E,$66,$7C,$B6,$48,$9C,$26,$2C,$B4,$B4,$FC,_
		  $E6,$FC,$E6,$7A,$40,$BC,$24,$BC,$00,$00,$48,$6C,$00,$D8,$00,$08,_
		  $00,$F6,$3E,$0E,$9E,$EE,$66,$FA,$26,$0A,$98,$26,$24,$16,$06,$1E,_
		  $E6,$F2,$00,$08,$2E,$1C,$04,$14,$00,$90,$0A,$02,$00,$00,$30,$FF],a

LOOKUP c,[$E2,$1E,$42,$1C,$4A,$16,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,_
		  $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,_
		  $00,$48,$40,$4A,$4A,$CE,$48,$40,$90,$24,$FE,$4A,$00,$02,$00,$84,_
		  $00,$00,$02,$02,$02,$02,$02,$00,$02,$02,$01,$01,$90,$02,$24,$0A,_
		  $42,$02,$4A,$00,$48,$00,$00,$02,$02,$48,$00,$90,$00,$A0,$30,$00,_
		  $02,$10,$12,$02,$48,$00,$84,$08,$B4,$A8,$84,$00,$30,$00,$A0,$00,_
		  $40,$02,$02,$02,$02,$02,$00,$02,$08,$0A,$00,$12,$00,$0A,$08,$02,_
		  $02,$02,$0A,$12,$00,$00,$04,$14,$B4,$42,$04,$90,$48,$26,$30,$FF],b

	endif
      
	if (a & %00000001) = 0 then
	   mdm_cursor_position =  mdm_cursor_position + 1
    ENDIF
	if (b & %00000001) = 0 then
	   mdm_cursor_position =  mdm_cursor_position + 1
	endif
	seg_a = ( seg_a & %00000001 ) | a	' don't overwrite the dots
	seg_b = ( seg_b & %00000001 ) | b

	if p = 1 then
		mdm_seg_1a = seg_a
		mdm_seg_1b = seg_b
	ENDIF
	if p = 2 then
		mdm_seg_2a = seg_a
		mdm_seg_2b = seg_b
	ENDIF
	if p = 3 then
		mdm_seg_3a = seg_a
		mdm_seg_3b = seg_b
	ENDIF
	if p = 4 then
		mdm_seg_4a = seg_a
		mdm_seg_4b = seg_b
	ENDIF
	if p = 5 then
		mdm_seg_5a = seg_a
		mdm_seg_5b = seg_b
	endif

	gosub mdm_output_byte
return


' ----------------------------------------------------------------------------
' write ascii character c to position 1xxxx on MDM-1.
' ----------------------------------------------------------------------------

mdm_write_1:	p = 1
				gosub mdm_write_p
return	


' ----------------------------------------------------------------------------
' write ascii character c to position x2xxx on MDM-1.
' ----------------------------------------------------------------------------

mdm_write_2:	p = 2
				gosub mdm_write_p
return	


' ----------------------------------------------------------------------------
' write ascii character c to position xx3xx on MDM-1.
' ----------------------------------------------------------------------------

mdm_write_3:	p = 3
				gosub mdm_write_p
return	


' ----------------------------------------------------------------------------
' write ascii character c to position xxx4x on MDM-1.
' ----------------------------------------------------------------------------

mdm_write_4:	p = 4
	   			gosub mdm_write_p
return	


' ----------------------------------------------------------------------------
' write ascii character c to position xxxx5 on MDM-1.
' ----------------------------------------------------------------------------

mdm_write_5:	p = 5
	   			gosub mdm_write_p
return	


' ----------------------------------------------------------------------------
' clear the display, set cursor position to 1
' ----------------------------------------------------------------------------

mdm_clear:	x = 0
			gosub mdm_all
			mdm_cursor_position = 1
			return

			
' ----------------------------------------------------------------------------
' set the display to all on, set cursor position to 1
' ----------------------------------------------------------------------------

mdm_set:	x = 1
			gosub mdm_all
			mdm_cursor_position = 1
			return

			
' ----------------------------------------------------------------------------
' write acii character c to cursor position, reset by mdm_clear.  Allows TTY
' style message to move scroll
' ----------------------------------------------------------------------------

mdm_write:	if mdm_cursor_position > 5 then
			   mdm_seg_1a = mdm_seg_2a
			   mdm_seg_1b = mdm_seg_2b
			   mdm_seg_2a = mdm_seg_3a
			   mdm_seg_2b = mdm_seg_3b
			   mdm_seg_3a = mdm_seg_4a
			   mdm_seg_3b = mdm_seg_4b
			   mdm_seg_4a = mdm_seg_5a
			   mdm_seg_4b = mdm_seg_5b
			   mdm_cursor_position = 5
			endif
			gosub mdm_write_p
			PAUSE 300
			return

			
' ----------------------------------------------------------------------------
' initialise the pin directions
' ----------------------------------------------------------------------------

mdm_init:	low mdm_pin_load
			low mdm_pin_data
			low mdm_pin_clock
			return



' ----------------------------------------------------------------------------
' Show a scrolling message by writing TTY-wise and rotating
' ----------------------------------------------------------------------------

show_hello:
    gosub mdm_clear
    mdm_cursor_position = 5
    For z = 0 to 2
        c = "H"
        gosub mdm_write
        c = "E"
        gosub mdm_write
        c = "L"
        gosub mdm_write
        c = "L"
        gosub mdm_write
        c = "0"
        gosub mdm_write
        c = " "
        gosub mdm_write
        c = "W"
        gosub mdm_write
        c = "O"
        gosub mdm_write
        c = "R"
        gosub mdm_write
        c = "L"
        gosub mdm_write
        c = "D"
        gosub mdm_write
        c = " "
        gosub mdm_write
    next
return

' ----------------------------------------------------------------------------
' Show all characters
' ----------------------------------------------------------------------------

show_ascii:
For z = 0 to 127
		c = (z Dig 2) + "0"
        Gosub mdm_write_1
        c = (z Dig 1) + "0"
        Gosub mdm_write_2
        c = (z Dig 0) + "0"
        Gosub mdm_write_3
        c = "="
        Gosub mdm_write_4
        c = ","
        Gosub mdm_write_5
        c = z
        Gosub mdm_write_5
        PAUSE 500
        next
return


' ----------------------------------------------------------------------------
' Display 12:34.5 by writing to individual positions
' ----------------------------------------------------------------------------

show_12345:
    Gosub mdm_clear
	c = "1"
    Gosub mdm_write_1
	c = "2"
    Gosub mdm_write_2
	c = ":"
    Gosub mdm_write_3
	c = "3"
    Gosub mdm_write_3
	c = "4"
    Gosub mdm_write_4
	c = "."
    Gosub mdm_write_5
	c = "5"
    Gosub mdm_write_5
    PAUSE 2000
return



